<xsl:transform
  xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
  version="1.0"
  >

  <xsl:key name="authkey" match="ITEM" use="AUTHOR"/>
  <xsl:key name="codekey" match="CATEGORY" use="@CODE"/>
  <xsl:decimal-format name="comma" decimal-separator="," grouping-separator="."/>

  <!--xsl:include href="books-attsets.xsl"/-->

  <xsl:variable name="categories" select="//CATEGORY"/>
  <xsl:variable name="now" xmlns:Date="/java.util.Date">
    <xsl:value-of select="Date:toString(Date:new())"/>
  </xsl:variable>

  <xsl:param name="top-author">Bonner</xsl:param>

  <xsl:preserve-space elements="AUTHOR TITLE"/>

  <xsl:template match="/">
    <HTML>
      <xsl:comment>Generated at <xsl:value-of select="$now"/></xsl:comment>

      <xsl:call-template name="header">
        <xsl:with-param name="title" select="'Book List'"/>
      </xsl:call-template>
      <BODY>
	<xsl:element name="div" use-attribute-sets="attset2">
	  <xsl:apply-templates/>
	</xsl:element>
      </BODY>
    </HTML>
  </xsl:template>

  <xsl:template name="header" xml:space="preserve">
    <xsl:param name="title" select="'Default Title'"/>
    <HEAD>
      <TITLE><xsl:value-of select="$title"/> - <xsl:value-of select="$now"/></TITLE></HEAD>
  </xsl:template>

  <xsl:template match="BOOKLIST">

    <H2>List of categories</H2>
    <xsl:apply-templates select="$categories">
      <xsl:with-param name="p2"><i>35</i></xsl:with-param>
    </xsl:apply-templates>

    <H2>This week's top author is <xsl:value-of select="$top-author"/></H2>
    <xsl:variable name="top-authors-books" select="key('authkey', $top-author)"/>
    
    <p>We stock the following <xsl:value-of select="count($top-authors-books)"/> books by this author:</p>
    <ul>
      <xsl:for-each select="$top-authors-books">
        <li><xsl:value-of select="TITLE"/></li>
      </xsl:for-each>
    </ul>
    
    <p>But this one is the first</p>
    <xsl:for-each select="$top-authors-books[1]">
      <p><xsl:value-of select="TITLE"/> costing $<xsl:value-of select="PRICE"/></p>
    </xsl:for-each>


    <p>This author has written books in the following categories:</p>
    <ul>
      <xsl:for-each select="key('codekey', $top-authors-books/@CAT)/@DESC">
        <li><xsl:value-of select="."/></li>
      </xsl:for-each>
    </ul>

    <p>The average price of these books is: <xsl:text/>
      <xsl:value-of select="format-number(
	sum($top-authors-books/PRICE)
	div count($top-authors-books),
	'$####.00')"/>
    </p>

    <H2>A complete list of books, sorted by author</H2>
    <xsl:for-each select="BOOKS/ITEM">
      <xsl:sort select="AUTHOR"/>
      <xsl:apply-templates select="." mode="by-author"/>
    </xsl:for-each>

    <H2>A complete list of books, sorted by category</H2>
    <xsl:for-each select="BOOKS/ITEM">
      <xsl:sort select="@CAT"/>
      <xsl:apply-templates select="." mode="by-category"/>
    </xsl:for-each>

  </xsl:template>   

  <xsl:template match="ITEM" mode="by-author">
    <h3>AUTHOR: <xsl:value-of select="AUTHOR"/></h3>
    <TABLE>
      <TR><TD WIDTH="100" VALIGN="TOP"></TD>
	<TD>
	  TITLE: <xsl:value-of select="TITLE"/><BR/>
	  CATEGORY: <xsl:value-of select="id(@CAT)/@DESC" />
	  (<xsl:value-of select="@CAT" />)
	</TD></TR>
    </TABLE><HR/>
  </xsl:template>

  <xsl:template match="ITEM" mode="by-category">
    <h3>CATEGORY: <xsl:value-of select="id(@CAT)/@DESC" /></h3>
    <OL>
      <LI>AUTHOR: <xsl:value-of select="AUTHOR"/><BR/>
	TITLE: <xsl:value-of select="TITLE"/></LI>                
    </OL><HR/>
  </xsl:template>

  <xsl:template match="CATEGORY" >
    <H4>CATEGORY <xsl:number value="position()" format="I"/></H4>
    <TABLE>
      <xsl:for-each select="@*">
        <TR>
	  <TD><xsl:value-of select="name(.)"/></TD>
	  <TD><xsl:value-of select="."/></TD>
        </TR>
      </xsl:for-each>
    </TABLE>
    <HR/>
  </xsl:template>

  <xsl:template match="@*">
    <xsl:copy/>
  </xsl:template>

  <xsl:template match="processing-instruction()">
    <p>Default PI template called</p>
    Name = <xsl:value-of select="name()"/>
    Value = <xsl:value-of select="."/>
    Copy: <xsl:copy/>
  </xsl:template>

  <xsl:template match="processing-instruction('testpi1')" priority="2">
    <p>PI template for testpi1 called:
      number (any) = <xsl:number level="any"/>
      number (single) = <xsl:number level="single"/>
      number (multiple) = <xsl:number level="multiple" count="node()"/>
    </p>
  </xsl:template>

  <xsl:template match="id('C')" priority="93.7">
    Found category C!
  </xsl:template>

</xsl:transform>	
